@@ -71,33 +71,18 @@ namespace ASGE
     int window_width{ 800 }; /**< The width of the GL Window to create. */                   // NOLINT
     int window_height{ 640 }; /**< The height of the GL Window to create. */                 // NOLINT
     int window_bpp{ 32 }; /**< The Bits Per Pixel (BPP) of the window. */                    // NOLINT
-    int msaa_level{ 1 }; /**< MSAA can help smooth edges when using rotated views. */        // NOLINT
-    int fps_limit{ 60 }; /**< The engine will attempt to never exceed this tick rate. */    // NOLINT
-    int fixed_ts{ fps_limit * 2 }; /**< The delta between fixed time-steps. */              // NOLINT
+    int msaa_level{ 32 }; /**< MSAA can help with edges, but be lowered if FPS is poor. */   // NOLINT
+    int fps_limit{ 60 }; /**< The engine will attempt to never exceed this tick rate. */     // NOLINT
+    int fixed_ts{ fps_limit * 2 }; /**< The delta between fixed time-steps. */               // NOLINT
     int anisotropic{ 16 }; /**< Improves filtering at oblique angles. Not useful for 2D. */  // NOLINT
 
     std::string write_dir{}; /**< The default write directory for ASGE IO. */
-    std::string window_title{ "My ASGE Game" }; /**< The window title. */
+    std::string game_title{ "My ASGE Game" }; /**< The window title. */
     WindowMode mode{ WindowMode::WINDOWED }; /**< The window mode i.e. Fullscreen. */
     MagFilter mag_filter {MagFilter::LINEAR}; /**< Textures will use this filter by default. */
     Vsync vsync {Vsync::ADAPTIVE};
   };
 
-
-  /// TODO: fix these clang warnings
-  #ifdef __clang__
-  #pragma clang diagnostic push
-  #pragma clang diagnostic ignored "-Wglobal-constructors"
-  #pragma clang diagnostic ignored "-Wexit-time-destructors"
-  #endif
-
-  /// The lazy way of grabbing the game's settings
-  inline GameSettings SETTINGS; //NOLINT
-
-  #ifdef __clang__
-  #pragma clang diagnostic pop
-  #endif
-
 }  // namespace ASGE
 #endif // ASGE_GAMESETTINGS_HPP
 
