@@ -71,6 +71,20 @@ namespace ASGE
         RGBA = 4 /**< There are RGB plus alpha channels present .*/
     };
 
+    /**
+     * Min filters used when scaling textures downwards
+     */
+    enum class MinFilter : uint32_t
+    {
+      LINEAR ,
+      LINEAR_MIPMAP_NEAREST,
+      LINEAR_MIPMAP_LINEAR,
+      NEAREST,
+      NEAREST_MIPMAP_LINEAR,
+      NEAREST_MIPMAP_NEAREST
+    };
+
+
     /**
      * @brief modes that controlling the UV texture wrapping
      *
@@ -128,6 +142,14 @@ namespace ASGE
 		*/
     virtual void updateMagFilter(MagFilter filter) = 0;
 
+    /**
+		* Sets the filtering used for texture minification.
+		* Allows the type of filtering applied when
+    * sampling the texture under minification to be changed.
+		* @param filter The minification filter to apply.
+     */
+    virtual void updateMinFilter(MinFilter filter) = 0;
+
     /**
      * @brief Controls how the UV coordinates are wrapped.
      *
