@@ -10,7 +10,7 @@
 //  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 //  SOFTWARE.
 
-#include <Engine/Font.hpp>
+#include "Font.hpp"
 int ASGE::Font::pxWide(const std::string& string) const
 {
   return static_cast<int>(pxWide(string, 1.0));
@@ -30,3 +30,8 @@ int ASGE::Font::pxHeight(const char* ch, float scale) const
 {
   return static_cast<int>(pxHeight(std::string(ch), scale));
 }
+
+std::tuple<float, float> ASGE::Font::boundsY(const char* ch, float scale) const
+{
+  return boundsY(std::string(ch), scale);
+}
