@@ -12,10 +12,21 @@
 
 #include "CGLSpriteRenderer.hpp"
 #include "GLRenderer.hpp"
+#include "Logger.hpp"
+#include "OpenGL/GLAtlas.hpp"
+#include "OpenGL/GLFontSet.hpp"
+#include "OpenGL/GLSprite.hpp"
 
-#include <OpenGL/GLAtlas.hpp>
-#include <OpenGL/GLFontSet.hpp>
-#include <OpenGL/GLSprite.hpp>
+namespace ASGE
+{
+  namespace
+  {
+    struct SHADER_DATA
+    {
+      glm::mat4 projection;
+    };
+  }
+}
 
 void ASGE::CGLSpriteRenderer::checkForErrors() const
 {
@@ -39,11 +50,11 @@ ASGE::SHADER_LIB::GLShader* ASGE::CGLSpriteRenderer::initShader(
   return nullptr;
 }
 
-bool ASGE::CGLSpriteRenderer::bindShader(GLuint shader_id, GLuint /*start_idx*/) noexcept
+bool ASGE::CGLSpriteRenderer::bindShader(GLuint shader_id, GLfloat distance) noexcept
 {
   shader_id == 0 ? shader_id = getBasicSpriteShaderID() : shader_id;
 
-  if(active_shader == nullptr || active_shader->getShaderID() != shader_id)
+  if(active_shader == nullptr || current_loaded_shader != shader_id)
   {
     auto iter = std::find(GLRenderer::shaders.begin(), GLRenderer::shaders.end(), shader_id);
     if (iter == GLRenderer::shaders.end())
@@ -53,6 +64,13 @@ bool ASGE::CGLSpriteRenderer::bindShader(GLuint shader_id, GLuint /*start_idx*/)
 
     active_shader = &(*iter);
     active_shader->use();
+    current_loaded_shader = shader_id;
+  }
+
+  //TODO: move towards a more agnostic way of applying uniforms
+  if(distance)
+  {
+    active_shader->getUniform("distance_factor")->set(distance);
   }
 
   return true;
@@ -88,7 +106,8 @@ bool ASGE::CGLSpriteRenderer::bindTexture(GLuint texture_id)
 {
   if (current_loaded_texture != texture_id)
   {
-    glBindTexture(GL_TEXTURE_2D, texture_id);
+    glActiveTexture(GL_TEXTURE0);
+    GLVMSG("Binding Texture", glBindTexture, GL_TEXTURE_2D, texture_id);
     current_loaded_texture = texture_id;
     return true;
   }
@@ -105,6 +124,7 @@ ASGE::CGLSpriteRenderer::~CGLSpriteRenderer()
   if (glfwGetCurrentContext() != nullptr)
   {
     glDeleteBuffers(1, &vertex_buffer);
+    glDeleteBuffers(1, &shader_data_location);
   }
 }
 
@@ -177,13 +197,13 @@ void ASGE::CGLSpriteRenderer::generateUvData(
     uvs[i] = uv_y;
   }
 
-  if (sprite.isFlippedOnX())
+  if (sprite.isFlippedOnX() || sprite.isFlippedOnXY())
   {
-    std::swap(uvs[0], uvs[4]);
-    std::swap(uvs[8], uvs[12]);
+    std::swap(uvs[0], uvs[12]);
+    std::swap(uvs[4], uvs[8]);
   }
 
-  if (sprite.isFlippedOnY())
+  if (sprite.isFlippedOnY() || sprite.isFlippedOnXY())
   {
     std::swap(uvs[1], uvs[5]);
     std::swap(uvs[9], uvs[13]);
@@ -194,22 +214,20 @@ void ASGE::CGLSpriteRenderer::createCharQuad(
   const ASGE::GLCharRender& character, const ASGE::Colour& colour, ASGE::GPUQuad& quad) const
 {
   // locate the character and set x,y positions
-  auto& ch     = character.font->getAtlas()->getCharacter(character.ch);
-  GLfloat xpos = character.x + ch.Bearing.x * powf(character.scale, 2);
-  GLfloat ypos = character.y - ch.Bearing.y * powf(character.scale, 2);
-  // GLfloat ypos = character.y - ch.Bearing.y *(ch.Size.y - ch.Bearing.y);
+  const auto& ch = character.font->getAtlas()->getCharacter(character.ch);
+  float x_pos    = character.x + ch.Bearing.x * character.scale;
+  float y_pos    = character.y - ch.Bearing.y * character.scale;
 
   // generate the matrix
   auto& model_matrix = quad.position;
-  model_matrix       = { glm::mat4(1.f) };
+  model_matrix       = { glm::mat4(1.F) };
 
-  model_matrix = glm::translate(model_matrix, glm::vec3(xpos, ypos, 0.0f));
+  model_matrix = glm::translate(model_matrix, glm::vec3(x_pos, y_pos, 0.0F));
 
   GLfloat w = ch.Size.x * character.scale;
   GLfloat h = ch.Size.y * character.scale;
 
-  model_matrix =
-    glm::scale(model_matrix, glm::vec3(w * character.scale, h * character.scale, 1.0f));
+  model_matrix = glm::scale(model_matrix, glm::vec3(w, h,1.0F));
 
   // Calculate texture coords for each character
   quad.uv_data[0] = glm::vec4{ (float)ch.UV.x, (float)ch.UV.w, GPUQuad::PADDING }; // v1
@@ -246,3 +264,40 @@ ASGE::SHADER_LIB::GLShader* ASGE::CGLSpriteRenderer::activeShader()
 {
   return active_shader;
 }
+
+void ASGE::CGLSpriteRenderer::apply(ASGE::RenderState* state)
+{
+  if(active_render_state != state)
+  {
+    if(!active_render_state || active_render_state->projection != state->projection)
+    {
+      glBindBuffer(GL_UNIFORM_BUFFER, shader_data_location);
+      glBufferSubData(
+        GL_UNIFORM_BUFFER,
+        0,
+        sizeof(glm::mat4),
+        glm::value_ptr(state->projection));
+    }
+
+    if(!active_render_state || active_render_state->viewport != state->viewport)
+    {
+      auto& vp = state->viewport;
+      glViewport(vp.x, vp.y, vp.w, vp.h);
+    }
+    active_render_state = state;
+  }
+}
+
+void ASGE::CGLSpriteRenderer::setupGlobalShaderData()
+{
+  // setup shader data location
+  glGenBuffers(1, &shader_data_location);
+  glBindBuffer(GL_UNIFORM_BUFFER, shader_data_location);
+  glBufferData(GL_UNIFORM_BUFFER, sizeof(SHADER_DATA), nullptr, GL_DYNAMIC_DRAW);
+  glBindBufferBase(GL_UNIFORM_BUFFER, GLRenderConstants::PROJECTION_UBO_BIND, shader_data_location);
+}
+
+void ASGE::CGLSpriteRenderer::clearActiveRenderState()
+{
+  active_render_state = nullptr;
+}
