@@ -10,10 +10,13 @@
 //  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 //  SOFTWARE.
 
-#include "GLInput.hpp"
-#include "GLIncludes.hpp"
-#include "GLRenderer.hpp"
 #include <glm/gtc/matrix_transform.hpp>
+#include <filesystem>
+
+#include "FileIO.hpp"
+#include "GLIncludes.hpp"
+#include "GLInput.hpp"
+#include "GLRenderer.hpp"
 
 bool ASGE::GLInput::init(Renderer* renderer)
 {
@@ -23,9 +26,7 @@ bool ASGE::GLInput::init(Renderer* renderer)
     return false;
   }
 
-  projection_matrix = &glrenderer->getProjectionMatrix();
   window            = glrenderer->getWindow();
-  glfwSetWindowUserPointer(window, this);
 
   auto kFunc = [](GLFWwindow* window, int key, int scancode, int action, int mods) {
     auto event      = std::make_shared<KeyEvent>();
@@ -33,7 +34,9 @@ bool ASGE::GLInput::init(Renderer* renderer)
     event->scancode = scancode;
     event->action   = action;
     event->mods     = mods;
-    static_cast<GLInput*>(glfwGetWindowUserPointer(window))->sendEvent(EventType::E_KEY, event);
+
+    auto* input = static_cast<WindowData*>(glfwGetWindowUserPointer(window))->input;
+    input->sendEvent(EventType::E_KEY, event);
   };
 
   auto mButtonEvent = [](GLFWwindow* window, int button, int action, int mods) {
@@ -42,8 +45,7 @@ bool ASGE::GLInput::init(Renderer* renderer)
     event->action = action;
     event->mods   = mods;
 
-    auto* input = static_cast<GLInput*>(glfwGetWindowUserPointer(window));
-
+    auto* input = static_cast<WindowData*>(glfwGetWindowUserPointer(window))->input;
     input->getCursorPos(event->xpos, event->ypos);
     input->sendEvent(EventType::E_MOUSE_CLICK, event);
   };
@@ -53,8 +55,7 @@ bool ASGE::GLInput::init(Renderer* renderer)
     event->xpos = xpos;
     event->ypos = ypos;
 
-    auto* input = static_cast<GLInput*>(glfwGetWindowUserPointer(window));
-
+    auto* input = static_cast<WindowData*>(glfwGetWindowUserPointer(window))->input;
     input->getCursorPos(event->xpos, event->ypos);
     input->sendEvent(EventType::E_MOUSE_MOVE, event);
   };
@@ -63,8 +64,8 @@ bool ASGE::GLInput::init(Renderer* renderer)
     auto event     = std::make_shared<ScrollEvent>();
     event->xoffset = xoffset;
     event->yoffset = yoffset;
-    static_cast<GLInput*>(glfwGetWindowUserPointer(window))
-      ->sendEvent(EventType::E_MOUSE_SCROLL, event);
+    auto* input = static_cast<WindowData*>(glfwGetWindowUserPointer(window))->input;
+    input->sendEvent(EventType::E_MOUSE_SCROLL, event);
   };
 
   glfwSetKeyCallback(glrenderer->getWindow(), kFunc);
@@ -84,14 +85,20 @@ void ASGE::GLInput::getCursorPos(double& xpos, double& ypos) const
 
 void ASGE::GLInput::unProjectCursor(double& xpos, double& ypos) const
 {
-  GLint viewport[4];
-  glGetIntegerv(GL_VIEWPORT, viewport);
+  auto& resolution = static_cast<WindowData*>(glfwGetWindowUserPointer(window))->resolution;
 
-  glm::vec4 viewportdata = glm::vec4(viewport[0], viewport[1], viewport[2], viewport[3]);
-  glm::vec3 pos          = glm::vec3(xpos, viewportdata.w - ypos, 0.0F);
+  auto view  = resolution->view;
+  constexpr float min = std::numeric_limits<decltype(RenderQuad::z_order)>::min();
+  constexpr float max = std::numeric_limits<decltype(RenderQuad::z_order)>::max();
+  auto pm = glm::ortho<float>(view.min_x, view.max_x, view.max_y, view.min_y, min, max);
+
+  auto& viewport = resolution->viewport;
+
+  glm::vec4 viewportdata = glm::vec4(viewport.x, viewport.y, viewport.w, viewport.h);
+  glm::vec3 pos          = glm::vec3(xpos, viewportdata.y * 2 + viewportdata.w - ypos, 0.0F);
   glm::mat4 model        = glm::mat4(1.0F);
 
-  auto unprojected = glm::unProject(pos, model, *projection_matrix, viewportdata);
+  auto unprojected = glm::unProject(pos, model, pm, viewportdata);
   xpos             = unprojected[0];
   ypos             = unprojected[1];
 }
@@ -116,33 +123,80 @@ void ASGE::GLInput::setCursorMode(ASGE::MOUSE::CursorMode mode)
 
 ASGE::GamePadData ASGE::GLInput::getGamePad(int idx) const
 {
-  /* NOTE: This is based on the current GLFW 3.2.1 version
-   *  however, there are a great deal of improvements happening
-   *  in 3.3,  some of which which alter the gamepad API and
-   *  introduce haptic feedback. We should look to upgrade when
-   *  it's finally released.
-   */
-
-  if (glfwJoystickPresent(idx) == 0)
+  if (glfwJoystickIsGamepad(idx) == 0)
   {
-    GamePadData data(nullptr, nullptr, std::string("not connected").c_str(), idx, 0, 0);
+    GamePadData data(idx, std::string("not connected").c_str(), nullptr, nullptr);
     data.is_connected = false;
     return data;
   }
 
-  const auto* name = glfwGetJoystickName(idx);
+  const auto* name = glfwGetGamepadName(idx);
+  GLFWgamepadstate state;
+  glfwGetGamepadState(idx, &state);
+  const auto* axis_state   = state.axes;
+  const auto* button_state = state.buttons;
 
-  int axis_count           = 0;
-  int button_count         = 0;
-  const auto* axis_state   = glfwGetJoystickAxes(idx, &axis_count);
-  const auto* button_state = glfwGetJoystickButtons(idx, &button_count);
-
-  GamePadData data(axis_state, button_state, name, idx, axis_count, button_count);
+  GamePadData data(idx, name, axis_state, button_state);
   data.is_connected = true;
   return data;
 }
 
+ASGE::GamePadData ASGE::GLInput::getGamePad() const
+{
+  for(int idx=0; idx<GLFW_JOYSTICK_LAST; ++idx)
+  {
+    if(glfwJoystickIsGamepad(idx))
+    {
+      const auto* name = glfwGetGamepadName(idx);
+      GLFWgamepadstate state;
+      glfwGetGamepadState(idx, &state);
+      const auto* axis_state   = state.axes;
+      const auto* button_state = state.buttons;
+      GamePadData data(idx, name, axis_state, button_state);
+      data.is_connected = true;
+      return data;
+    }
+  }
+  return ASGE::GamePadData(0, std::string("not connected").c_str(), nullptr, nullptr);
+}
+
+std::vector<ASGE::GamePadData> ASGE::GLInput::getGamePads() const
+{
+  std::vector<ASGE::GamePadData> gamepads;
+  for(int idx=0; idx<GLFW_JOYSTICK_LAST; ++idx)
+  {
+    auto data = getGamePad(idx);
+    if(data.is_connected)
+    {
+      gamepads.emplace_back(std::move(data));
+    }
+  }
+  return gamepads;
+}
+
 void ASGE::GLInput::setCursorPos(double xpos, double ypos) const
 {
   glfwSetCursorPos(this->window, xpos, ypos);
 }
+
+void ASGE::GLInput::updateGamePadMappings(const std::filesystem::path &mappings_file)
+{
+  std::string file_contents;
+  ASGE::FILEIO::File file;
+  if(file.open(mappings_file.string(), ASGE::FILEIO::File::IOMode::READ))
+  {
+    const auto buffer = file.read();
+    file_contents = std::string(buffer.data.get(), buffer.length);
+  }
+
+  else if(!exists(mappings_file))
+  {
+    std::ifstream file(mappings_file.c_str());
+    file_contents = std::string((std::istreambuf_iterator<char>(file)), std::istreambuf_iterator<char>());
+  }
+
+  if (file_contents.empty())
+    return;
+
+  glfwUpdateGamepadMappings(file_contents.c_str());
+}
\ No newline at end of file
