@@ -75,7 +75,7 @@ void initGamesettings(py::module_ & module){
     )")
 
     .def_readwrite(
-      "window_title", &ASGE::GameSettings::window_title,
+      "window_title", &ASGE::GameSettings::game_title,
       R"(
           Each window has a title, customise yours.
 
@@ -104,16 +104,26 @@ void initGamesettings(py::module_ & module){
           :type: int
     )")
 
-      .def_readwrite(
-          "fps_limit", &ASGE::GameSettings::fps_limit,
-          R"(
-          Attempts to limit the game's FPS
+    .def_readwrite(
+      "msaa_level", &ASGE::GameSettings::msaa_level,
+      R"(
+          Controls the pacing of the fixed time-step update function.
 
-          :getter: Returns the FPS limit
-          :setter: Sets desired FPS cap
+          :getter: Returns the desired delta between update calls.
+          :setter: Sets the desired fixed time-step delta.
           :type: int
     )")
 
+    .def_readwrite(
+        "fps_limit", &ASGE::GameSettings::fps_limit,
+        R"(
+        Attempts to limit the game's FPS
+
+        :getter: Returns the FPS limit
+        :setter: Sets desired FPS cap
+        :type: int
+    )")
+
     .def_readwrite(
       "mag_filter",
       &ASGE::GameSettings::mag_filter,
@@ -139,7 +149,7 @@ void initGamesettings(py::module_ & module){
            "Present the game in full-screen, changing resolution if necessary.")
     .value("WINDOWED", ASGE::GameSettings::WindowMode::WINDOWED,
            "Present the game in a window.")
-    .value("BORDERLESS_WINDOW", ASGE::GameSettings::WindowMode::WINDOWED,
+    .value("BORDERLESS_WINDOW", ASGE::GameSettings::WindowMode::BORDERLESS_WINDOWED,
            "Present the game in a borderless window.")
     .value("BORDERLESS_FULLSCREEN", ASGE::GameSettings::WindowMode::BORDERLESS_FULLSCREEN,
            "Present the game in a fullscreen borderless window, maintaining the "
